fn lab_inv_f(ft: f32) -> f32 {
    let e = 216.0 / 24389.0;
    let k = 24389.0 / 27.0;
    let ft3 = ft * ft * ft;
    if ft3 > e {
        ft3
    } else {
        (116.0 * ft - 16.0) / k
    }
}

fn lab_f(t: f32) -> f32 {
    let e = 216.0 / 24389.0;
    let k = 24389.0 / 27.0;
    if t > e {
        t.powf(1.0 / 3.0)
    } else {
        (k * t + 16.0) / 116.0
    }
}

#[inline]
fn y_from_lstar(lstar: f32) -> f32 {
    100.0 * lab_inv_f((lstar + 16.0) / 116.0)
}

#[inline]
fn lstar_from_y(y: f32) -> f32 {
    lab_f(y / 100.0) * 116.0 - 16.0
}

pub fn ratio_of_ys(y1: f32, y2: f32) -> f32 {
    let lighter = y1.max(y2);
    let darker = y1.min(y2);
    (lighter + 5.0) / (darker + 5.0)
}

pub fn ratio_of_tones(a: f32, b: f32) -> f32 {
    let tone_a = a.clamp(0.0, 100.0);
    let tone_b = b.clamp(0.0, 100.0);
    ratio_of_ys(y_from_lstar(tone_a), y_from_lstar(tone_b))
}

pub fn lighter(tone: f32, ratio: f32) -> f32 {
    if tone < 0.0 || tone > 100.0 {
        return -1.0;
    }

    let dark_y = y_from_lstar(tone);
    let light_y = ratio * (dark_y + 5.0) - 5.0;
    let real_contrast = ratio_of_ys(light_y, dark_y);
    let delta = (real_contrast - ratio).abs();
    if real_contrast < ratio && delta > 0.04 {
        return -1.0;
    }

    let return_value = lstar_from_y(light_y) + 0.4;
    if return_value < 0.0 || return_value > 100.0 {
        return -1.0;
    }
    return_value
}

pub fn darker(tone: f32, ratio: f32) -> f32 {
    if tone < 0.0 || tone > 100.0 {
        return -1.0;
    }

    let light_y = y_from_lstar(tone);
    let dark_y = ((light_y + 5.0) / ratio) - 5.0;
    let real_contrast = ratio_of_ys(light_y, dark_y);

    let delta = (real_contrast - ratio).abs();

    if real_contrast < ratio && delta > 0.04 {
        return -1.0;
    }

    let return_value = lstar_from_y(dark_y) - 0.4;
    if return_value < 0.0 || return_value > 100.0 {
        return -1.0;
    }
    return_value
}

pub fn unsafe_lighter(tone: f32, ratio: f32) -> f32 {
    let safe_lighter = lighter(tone, ratio);
    if safe_lighter < 0.0 {
        100.0
    } else {
        safe_lighter
    }
}

pub fn unsafe_darker(tone: f32, ratio: f32) -> f32 {
    let safe_darker = darker(tone, ratio);
    if safe_darker < 0.0 {
        0.0
    } else {
        safe_darker
    }
}
