import cx from 'clsx';
import { isEqual } from 'lodash-es';
import { ChangeEvent, useEffect, useState } from 'react';
import styles from './ColorRangePicker.module.css';

type ColorRangePickerProps = {
  title: string;
  value?: number;
  unit?: string;
  onChange: (value: number) => void;
  gradient: string;
  min?: number;
  max: number;
  step?: number;
  valueProcess?: (value: number) => number;
};

export function ColorRangePicker({
  title,
  value = 0,
  unit,
  onChange,
  gradient,
  min = 0,
  max = 100,
  step = 1,
  valueProcess = (v) => v,
}: ColorRangePickerProps) {
  const [pickerValue, setPickerValue] = useState(value);
  const handlePickerChange = (evt: ChangeEvent<HTMLInputElement>) => {
    const value = Number(evt.target.value);
    setPickerValue(valueProcess(value));
    onChange?.(valueProcess(value));
  };

  useEffect(() => {
    if (!isEqual(value, pickerValue)) {
      setPickerValue(value);
    }
  }, [value]);

  return (
    <div className={styles.picker_container}>
      <div className={styles.title_row}>
        <label>{title}</label>
        <span>
          {valueProcess(pickerValue)}
          {unit}
        </span>
      </div>
      <input
        type="range"
        className={cx('picker')}
        value={pickerValue}
        onChange={handlePickerChange}
        min={min}
        max={max}
        step={step}
        style={{ background: gradient }}
      />
    </div>
  );
}
