import { Differ, OklchDifference } from 'color-module';
import { useMemo } from 'react';
import { useColorFunction } from '../../ColorFunctionContext';
import styles from './CompareLayout.module.css';
import { CompareMethodProps } from './share-props';

export function OklchCompare({
  basic = '000000',
  compare = '000000',
  mode = 'absolute',
}: CompareMethodProps) {
  const { colorFn } = useColorFunction();
  const defaultCompareResult: OklchDifference = useMemo(
    () => new OklchDifference(new Differ(0, 0), new Differ(0, 0), new Differ(0, 0)),
    [],
  );
  const differ = useMemo(() => {
    if (!colorFn) {
      return defaultCompareResult;
    }
    try {
      const diffFn = {
        absolute: colorFn.differ_in_oklch,
        relative: colorFn.relative_differ_in_oklch,
      };
      const diff = diffFn[mode](basic, compare);
      return diff;
    } catch (e) {
      console.error('[Compare in Oklch]', e);
    }
    return defaultCompareResult;
  }, [basic, compare, mode]);

  return (
    <div>
      <h6>Compare in Oklch Space</h6>
      <div className={styles.elements}>
        <div className={styles.element}>
          <div className={styles.element_name}>L</div>
          <div className={styles.element_values}>
            <span>{(differ.lightness.delta * 100).toFixed(2)}%</span>
            <span>
              {isNaN(differ.lightness.percent)
                ? '0.0000'
                : (differ.lightness.percent * 100).toFixed(2)}
              %
            </span>
          </div>
        </div>
        <div className={styles.element}>
          <div className={styles.element_name}>C</div>
          <div className={styles.element_values}>
            <span>{differ.chroma.delta.toFixed(4)}</span>
            <span>
              {isNaN(differ.chroma.percent) ? '0.0000' : (differ.chroma.percent * 100).toFixed(2)}%
            </span>
          </div>
        </div>
        <div className={styles.element}>
          <div className={styles.element_name}>H</div>
          <div className={styles.element_values}>
            <span>{differ.hue.delta.toFixed(2)}</span>
            <span>
              {isNaN(differ.hue.percent) ? '0.00' : (differ.hue.percent * 100).toFixed(2)}%
            </span>
          </div>
        </div>
      </div>
    </div>
  );
}
