import dayjs from 'dayjs';
import { isNil, set } from 'lodash-es';
import { useCallback, useEffect, useMemo } from 'react';
import { useNavigate, useParams } from 'react-router-dom';
import { EditableDescription } from '../components/EditableDescription';
import { EditableTitle } from '../components/EditableTitle';
import { SchemeSign } from '../components/SchemeSign';
import { MaterialDesign2SchemeStorage } from '../material-2-scheme';
import {
  MaterialDesign3DynamicSchemeStorage,
  MaterialDesign3SchemeStorage,
} from '../material-3-scheme';
import { SchemeContent } from '../models';
import { CorruptedScheme } from '../page-components/scheme/CorruptedScheme';
import { M2Scheme } from '../page-components/scheme/M2Scheme';
import { M3DynamicScheme } from '../page-components/scheme/M3DynamicScheme';
import { M3Scheme } from '../page-components/scheme/M3Scheme';
import { QScheme } from '../page-components/scheme/QScheme';
import { SwatchScheme } from '../page-components/scheme/SwatchScheme';
import { QSchemeStorage } from '../q-scheme';
import { useScheme, useUpdateScheme } from '../stores/schemes';
import { SwatchSchemeStorage } from '../swatch_scheme';
import styles from './SchemeDetail.module.css';

export function SchemeDetail() {
  const { id } = useParams();
  const scheme = useScheme(id);
  const navigate = useNavigate();
  const updateScheme = useUpdateScheme(id);

  const updateTitle = useCallback(
    (newTitle: string) => {
      updateScheme((prev) => {
        set(prev, 'name', newTitle);
        return prev;
      });
    },
    [id],
  );
  const updateDescription = useCallback(
    (newDescription: string | null) => {
      updateScheme((prev) => {
        set(prev, 'description', newDescription);
        return prev;
      });
    },
    [id],
  );
  const schemeContent = useMemo(() => {
    switch (scheme?.type) {
      case 'q_scheme':
        return <QScheme scheme={scheme as SchemeContent<QSchemeStorage>} />;
      case 'swatch_scheme':
        return <SwatchScheme scheme={scheme as SchemeContent<SwatchSchemeStorage>} />;
      case 'material_2':
        return <M2Scheme scheme={scheme as SchemeContent<MaterialDesign2SchemeStorage>} />;
      case 'material_3':
        return <M3Scheme scheme={scheme as SchemeContent<MaterialDesign3SchemeStorage>} />;
      case 'material_3_dynamic':
        return (
          <M3DynamicScheme scheme={scheme as SchemeContent<MaterialDesign3DynamicSchemeStorage>} />
        );
      default:
        return <CorruptedScheme />;
    }
  }, [scheme]);

  useEffect(() => {
    if (isNil(scheme)) {
      navigate('../not-found');
    }
  }, [scheme, navigate]);

  return (
    <div className={styles.scheme_detail_layout}>
      <EditableTitle title={scheme?.name} onChange={updateTitle} />
      <div className={styles.badge_layout}>
        <SchemeSign scheme={scheme?.type} />
        <div className={styles.create_time}>
          created at {dayjs(scheme?.createdAt).format('YYYY-MM-DD')}
        </div>
      </div>
      <EditableDescription content={scheme?.description} onChange={updateDescription} />
      {schemeContent}
    </div>
  );
}
